From a2133f5904ddb616b6578920bd2199f5accb484d Mon Sep 17 00:00:00 2001
From: Tristan Matthews <tmatth@videolan.org>
Date: Fri, 24 Apr 2015 15:55:32 -0400
Subject: [PATCH] preprocess: prevent rare overflow on overlap-add

Reported-by: Fabian Henze <flyser42@gmx.de>
---
 libspeexdsp/arch.h       | 3 +++
 libspeexdsp/preprocess.c | 2 +-
 libspeexdsp/resample.c   | 6 ------
 3 files changed, 4 insertions(+), 7 deletions(-)

diff --git a/libspeexdsp/arch.h b/libspeexdsp/arch.h
index 6b3bc5d..c2de991 100644
--- a/libspeexdsp/arch.h
+++ b/libspeexdsp/arch.h
@@ -101,6 +101,8 @@ typedef spx_word32_t spx_sig_t;
 #define SIG_SHIFT    14
 #define GAIN_SHIFT   6
 
+#define WORD2INT(x) ((x) < -32767 ? -32768 : ((x) > 32766 ? 32767 : (x)))
+
 #define VERY_SMALL 0
 #define VERY_LARGE32 ((spx_word32_t)2147483647)
 #define VERY_LARGE16 ((spx_word16_t)32767)
@@ -203,6 +205,7 @@ typedef float spx_word32_t;
 #define DIV32(a,b)     (((spx_word32_t)(a))/(spx_word32_t)(b))
 #define PDIV32(a,b)     (((spx_word32_t)(a))/(spx_word32_t)(b))
 
+#define WORD2INT(x) ((x) < -32767.5f ? -32768 : ((x) > 32766.5f ? 32767 : floor(.5+(x))))
 
 #endif
 
diff --git a/libspeexdsp/preprocess.c b/libspeexdsp/preprocess.c
index c080581..3053eb5 100644
--- a/libspeexdsp/preprocess.c
+++ b/libspeexdsp/preprocess.c
@@ -980,7 +980,7 @@ EXPORT int speex_preprocess_run(SpeexPreprocessState *st, spx_int16_t *x)
 
    /* Perform overlap and add */
    for (i=0;i<N3;i++)
-      x[i] = st->outbuf[i] + st->frame[i];
+      x[i] = WORD2INT(ADD32(EXTEND32(st->outbuf[i]), EXTEND32(st->frame[i])));
    for (i=0;i<N4;i++)
       x[N3+i] = st->frame[N3+i];
 
diff --git a/libspeexdsp/resample.c b/libspeexdsp/resample.c
index 4940a64..4e47d67 100644
--- a/libspeexdsp/resample.c
+++ b/libspeexdsp/resample.c
@@ -83,12 +83,6 @@ static void speex_free (void *ptr) {free(ptr);}
 #define M_PI 3.14159265358979323846
 #endif
 
-#ifdef FIXED_POINT
-#define WORD2INT(x) ((x) < -32767 ? -32768 : ((x) > 32766 ? 32767 : (x)))  
-#else
-#define WORD2INT(x) ((x) < -32767.5f ? -32768 : ((x) > 32766.5f ? 32767 : floor(.5+(x))))  
-#endif
-               
 #define IMAX(a,b) ((a) > (b) ? (a) : (b))
 #define IMIN(a,b) ((a) < (b) ? (a) : (b))
 
-- 
2.1.4

